var TextFader = function () {

    /**
     * Default settings for the fader
     * @type {{selector: string, content: array, delay: int, fade: int, loop: boolean}}
     */
    var defaultSettings = {
        selector: '.textfade',
        content: ['This is text one', 'Here is another text', 'And this is the last one'],
        delay: 5000, // Milliseconds
        fade: 500, // Milliseconds
        loop: true
    };

    /**
     * Default settings for restarting the fader
     * @type {{startAt: int}}
     */
    var defaultRestartSettings = {
        startAt: 1
    };

    /**
     * Strings the fader should display
     * @type {array}
     */
    var content;

    /**
     * Index of the array the fader is currently at
     * @type {int}
     */
    var index;

    /**
     * Id for the interval object
     * @type {int}
     */
    var timer;

    /**
     * Updates the text in the fader
     * @param {object} object1
     * @param {object} object2
     */
    var mergeSettings  = function(object1, object2) {
        for (var attrname in object1) {
            if(object2.hasOwnProperty(attrname)) {
                object1[attrname] = object2[attrname];
            }
        }
    };

    /**
     * Updates the text in the fader
     */
    var update = function() {

        //Changes the text (with fade)
        $(defaultSettings.selector).animate({'opacity': 0}, defaultSettings.fade, function () {
            $(defaultSettings.selector).text(content[index].toString());
        }).animate({'opacity': 1}, defaultSettings.fade);

        //Increases index by one for next update cycle
        index++;

        //Is the fader configured as loop?
        if(defaultSettings.loop) {
            if(index >= content.length) {
                //Set the index back to 0 to keep the loop going
                index = 0;
            }
        } else {
            if(index >= content.length - 1) {
                //Remove the interval
                clearInterval(timer);
            }
        }
    };

    /**
     * Initializes the fader
     * @param {object} [settings] Object that contain overrides for the default settings
     */
    var __init = function(settings) {
        //Merge the init settings with the default settings
        mergeSettings(defaultSettings, settings || {});

        index = 0;
        content = defaultSettings.content;

        //Set element text to first string in the array
        $(defaultSettings.selector).text(content[0]);

        //Start update function that updates the text
        timer = setInterval(update, defaultSettings.delay);
    };


    /**
     * Restarts the fader
     * @param {object} [settings] Object that contain overrides for the default settings
     */
    var __restart = function(settings) {
        //If the update function is still running, this will stop it
        clearInterval(timer);

        //Merge the init settings with the default settings
        mergeSettings(defaultRestartSettings, settings || {});

        //Because arrays start at 0
        index = defaultRestartSettings.startAt - 1;

        //Set element text to first string in the array
        $(defaultSettings.selector).text(content[index]);

        //Restart update function that updates the text
        timer = setInterval(update, defaultSettings.delay);
    };

    // Return public funtions
    return {
        init: __init,
        restart: __restart,
    };

};